import QtQuick 1.1
import qb.components 1.0

/**
	Main screen of thermostat program. Shows program schedule for whole week.
 */
Screen {
	id: programScreen

	screenTitleIconUrl: "drawables/program.png"
	screenTitle: qsTr("Program")
	/// program enabled state
	property bool programEnabled: app.programEnabled
	/// current time arrow icon
	property url currentTimeArrowIconUrl: "./drawables/littlearrow.png"

	function init() {
		app.thermostatProgramChanged.connect(p.programChanged);
		app.thermostatProgramLoaded.connect(p.programLoaded);
	}

	property variant thermStates: app.thermStates

	/**
	 * Modify program day option selected
	 * @param type:int index of the selected day (Sunday == 0)
	 */
	signal modifyProgramDay(int day);

	QtObject {
		id: p
		
		// design height of the program for day
		property int programDayHeight: 220
		// pixels between hour lines. Also used to calculate program block height in pixels based on start and end time
		property real pixelsPerHour: 9
		// pixel count per 10 minutes == pixelsPerHour / 6. Program start/end time can be set in 10 minutes multiple
		property real pixelsPer10Minuts: 1.5
		// string for "Copy" current program button
		property string copyProgramBtnText: qsTr("Copy");
		// string for "Modify" current program button
		property string modifyProgramBtnText: qsTr("Modify");
		// type of the change done to save: 0 - no change, 1 - edit day program, 2 - copy day program
		property int savingProgram: 0
		// argument for saving the program change
		property variant saveProgramArgs

		// formats program start time in h:mm format for time scale. Program block with start hours and start minutes as input
		function formatTime(programBlock) {
			var m = programBlock.startMin < 10 ? "0" + programBlock.startMin : programBlock.startMin;
			return programBlock.startHour + ":" + m;
		}

		// determins if the program block start time at index @idx of the day program @program shoudl be displayed below (default) or
		// above the line that indicates that time. see timeScale component description
		function getTimeTextPos(program, idx) {
			if (idx < 0 || idx >= program.length)
				return timeScale.posHidden;

			var pos = timeScale.posBelow;
			if (idx + 1 < program.length) {
				if (p.getProgramStartInMinutes(program[idx + 1]) - p.getProgramStartInMinutes(program[idx]) <= 60) {
					pos = timeScale.posAbove;
				}
			}

			return pos;
		}

		// calculates program block start time in minutes from 0:00
		function getProgramStartInMinutes(programBlock) {
			return (programBlock.startHour * 60) + programBlock.startMin;
		}

		//calculates program block height in pixels based on its duration (next program start minus this program start) in tens of minutes and pixels per tens of minute
		function getProramHeightInPx(startTime, endTime) {
			return ((endTime - startTime) / 10) * p.pixelsPer10Minuts;
		}

		// calculates program block height for the last program of the day
		function getProramHeightInPxTillEnd(startTime) {
			var heigthFromStart = getProramHeightInPx(0, startTime);
			return p.programDayHeight - heigthFromStart;
		}

		// updates current time line position - program height from the day start till "now"
		function updateCurrentTimeLine() {
			var now = new Date();
			currentTime.offset = p.getProramHeightInPx(0, now.getHours()*60 + now.getMinutes());
		}

		// updates current time line update timer interval to update at next tens of minutes
		function updateCurrentTimeInterval() {
			var now = new Date();
			currentTimeTimer.interval = ((Math.floor((now.getMinutes() + 10)/ 10) * 10) - now.getMinutes()) * 60*1000;
		}

		// handles displaying/hiding the current time line and starting/stopping of the update timer
		function setCurrentTimeEnabled(state) {
			if (state) {
				updateCurrentTimeLine();
				updateCurrentTimeInterval();
				currentTimeTimer.start();
			}
			else {
				currentTimeTimer.stop();
			}
			currentTime.visible = state;
		}

		// handles program enabled/disabled change. e.g. on disable: hide current time line, set the opacity, hide modify buttons
		function programEnabledChanged() {
			timeScale.state = programEnabled === true ? "up" : "disabled";
			p.updateTimeScale();
			setCurrentTimeEnabled(programEnabled);
			modifyButtons.visible = programEnabled;
		}

		// updates time scale to actually selected day program. see timeScale component description
		function updateTimeScale() {
			textModel.clear();
			var day = programSchedule.selectedDay;
			var program;
			if (programEnabled && app.thermostatProgram && app.thermostatProgram[day]) {
				program = app.thermostatProgram[day];
			} else {
				return;
			}

			//the first item in the day program is the last program block from the day before (can continue till today)
			for (var i = 1; i < program.length; i++) {
				var height = 0;
				var pos = timeScale.posHidden;
				var posNext = timeScale.posHidden;

				if (i === 1) {
					//if the first program block this day continues from the day before, create artifical item for this
					//block (0:00 till next block start == this block end), hide start time line
					height = p.getProramHeightInPx(0, p.getProgramStartInMinutes(program[i]));
					pos = timeScale.posHidden;
					posNext = p.getTimeTextPos(program, i);
					var firstItem = {
						"timeTextUpper": "",
						"timeTextBottom": posNext === timeScale.posAbove ? p.formatTime(program[i]) : "",
						"programHeight": height,
						"upperVisible": pos === timeScale.posBelow,
						"bottomVisible": posNext === timeScale.posAbove,
						"startLineVisible": false
					};

					textModel.append(firstItem);
				}

				if (i + 1 < program.length) {
					height = p.getProramHeightInPx(p.getProgramStartInMinutes(program[i]), p.getProgramStartInMinutes(program[i + 1]));
				}
				else {
					height = p.getProramHeightInPxTillEnd(p.getProgramStartInMinutes(program[i]));
				}
				pos = p.getTimeTextPos(program, i);
				posNext = p.getTimeTextPos(program, i + 1);
				var item = {
					"timeTextUpper": pos === timeScale.posBelow ? p.formatTime(program[i]) : "",
					"timeTextBottom": posNext === timeScale.posAbove ? p.formatTime(program[i + 1]) : "",
					"programHeight": height,
					"upperVisible": pos === timeScale.posBelow,
					"bottomVisible": pos === timeScale.posBelow ? posNext === timeScale.posAbove : false,
					"startLineVisible": true
				};

				textModel.append(item);
			}
		}

		function getToday() {
			var now = new Date();
			return now.getDay();
		}

		// maps index of the program state (0..3: comfort, home, sleep, away) to extendend program state as it is in dataset (vacation, undefined in addition)
		function programStateIdxToAppStateIdx(stateIdx) {
			var idx = app.thermStateUndef
			switch (stateIdx) {
				case 0: idx = app.thermStateAway; break;
				case 1: idx = app.thermStateSleep; break;
				case 2: idx = app.thermStateActive; break;
				case 3: idx = app.thermStateRelax; break;
			}

			return idx;
		}

		// retreives program state temperature from the dataset based on the limited (0..3: comfort, home, sleep, away) index
		function getStateTemp(stateIdx) {
			return thermStates[app.thermStatesMap[programStateIdxToAppStateIdx(stateIdx)]].temperature;
		}

		/// formats temperature value to text including unit sign
		function formatTemp(temp) {
			return i18n.number(temp, 1) + "°";
		}

		function programChanged() {
			programSchedule.populate();
			updateTimeScale();
		}

		function programLoaded() {
			if (p.savingProgram > 0)
				app.waitPopup.hide();
			p.savingProgram = 0;
		}
	}

	function saveProgram(changeType, args) {
		p.savingProgram = changeType;
		p.saveProgramArgs = args;
		app.waitPopup.show();
		saveTimer.start();
	}

	onShown: {
		p.programEnabledChanged();
		programSchedule.selectDay(p.getToday());
	}

	Timer {
		id: saveTimer
		interval: 100
		running: false
		repeat: false
		onTriggered: {
			if (p.savingProgram == 1)
				app.saveEditedProgram();
			else if (p.savingProgram == 2)
				app.saveCopyProgramDay(p.saveProgramArgs.fromDay, p.saveProgramArgs.toDays);
		}
	}

	onHidden: {
		currentTimeTimer.stop();
	}

	onProgramEnabledChanged: {
		p.programEnabledChanged();
	}

	ThermostatProgramSchedule {
		id: programSchedule
		programEnabled: programScreen.programEnabled
		anchors {
			top: parent.top
			topMargin: 55
			left: parent.left
			leftMargin: 90
		}

		onSelectedDayChanged: {
			p.updateTimeScale();
		}
	}

	/// Timscale of the currently selected day program. Displayed using 2 Repeaters. One static for displaying short line for every hour - 25 lines, including 0:00.
	/// The second one to dynamicly create and display the start time of the program block. By default the time text is displayed below the line
	/// of the start time line but when two lines are an hour or less away from each other, the top one has the time placed above the line. When multiple lines are
	/// placed together, the middle ones do not show the time.
	/// There is one item in the Repeater for each program block and each item contains a long line at the top representing the start time and two texts - upper and bottom text.
	/// Upper text represents time text for current block should it be displayed below the line. The bottom text represents time for the next block should it be
	/// displayed above the line. The bottom time text can only be visible if the current block start time is below. Unused text is not visible.
	Item {
		id: timeScale

		property int linesCount: 25
		property int shortLength: 20
		property int longLenght: 55

		property int posAbove: 1
		property int posHidden: 0
		property int posBelow: -1
		property color scaleColor: colors.psTimeScale
		property real scaleOpacity: 1

		anchors {
			top: parent.top
			topMargin: 55+36+8
			right: programSchedule.left
			rightMargin: 7
		}
		width: longLenght
		height: p.programDayHeight

		ListModel {
			id: textModel
		}

		Column {
			width: parent.width
			height: parent.height

			Repeater {
				id: linesRepeater
				model: timeScale.linesCount
				Item {
					id: lineWrap

					width: parent.width
					height: p.pixelsPerHour

					Rectangle {
						id: hourLine
						color: timeScale.scaleColor
						opacity: timeScale.scaleOpacity
						height: 1
						width: timeScale.shortLength
						anchors.right: parent.right
					}
				}
			}
		}

		Column {
			width: parent.width
			height: parent.height

			Repeater {
				id: textRepeater
				model: textModel

				Item {
					id: textWrap

					width: timeScale.width
					height: programHeight

					Rectangle {
						id: programStartTime
						color: timeScale.scaleColor
						opacity: timeScale.scaleOpacity
						height: 1
						width: timeScale.longLenght
						anchors.top: parent.top
						visible: startLineVisible
					}

					Text {
						id: textRepeaterUpperText
						anchors.left: parent.left
						anchors.top: parent.top
						color: timeScale.scaleColor
						opacity: timeScale.scaleOpacity
						font {
							family: qfont.regular.name
							pixelSize: 10
						}
						visible: upperVisible
						text: timeTextUpper
					}

					Text {
						id: textRepeaterBottomText
						anchors.left: parent.left
						anchors.bottom: parent.bottom
						color: timeScale.scaleColor
						opacity: timeScale.scaleOpacity
						font {
							family: qfont.regular.name
							pixelSize: 10
						}
						visible: bottomVisible
						text: timeTextBottom
					}
				}
			}
		}

		states: [
			State {
				name: "up"
			},
			State {
				name: "disabled"
				PropertyChanges {
					target: timeScale
					scaleOpacity: 0.3
				}
			}
		]
	}

	/// The time line showing what the curent time is. Hidden when the program is disabled. Updates its position at evry 10s of minutes.
	Item {
		id: currentTime

		property real offset: 0;

		width: 635
		height: p.programDayHeight
		anchors.top: timeScale.top
		anchors.left: timeScale.left
		z: 0.5

		Rectangle {
			id: currentTimeLine

			color: colors.psCurrentTimeLine
			width: 600
			height: 1
			anchors.top: parent.top
			anchors.left: parent.left
			anchors.leftMargin: 35
			anchors.topMargin: currentTime.offset
		}

		Image {
			id: currentTimeArrow

			anchors.verticalCenter: currentTimeLine.verticalCenter
			anchors.right: parent.right
			anchors.rightMargin: 600
			source: currentTimeArrowIconUrl
		}

		Timer {
			id: currentTimeTimer
			running: false
			repeat: true;
			onTriggered: {
				p.updateCurrentTimeLine();
				p.updateCurrentTimeInterval();
			}
		}
	}

	/// buttons to "Copy" and "Modify" currently selected day program. Move with the selection of the day. Hidden when program disabled
	Item {
		id: modifyButtons

		// Sunday is index 0, for the left margin calculation it has to be converted
		property int selectedDay: programSchedule.selectedDay;

		width: 75*7 + 8*6 + 2*10 // 75*7 - program width, 8*6 - program spcacing, 2*10 - border
		height: 8*3 + 36*2 //8*3 - spacing, 36*2 - button heignt
		anchors.bottom: parent.bottom
		anchors.left: programSchedule.left
		anchors.leftMargin: -10

		StandardButton {
			id: btnCopyProgram
			width: 95
			height: 36
			topClickMargin: 4
			anchors {
				bottom: parent.bottom
				bottomMargin: 8
				left: parent.left
				leftMargin: (modifyButtons.selectedDay == 0 ? 6 : modifyButtons.selectedDay - 1) * (8+75)
			}

			text: p.copyProgramBtnText

			onClicked: {
				stage.openFullscreen(app.copyProgramDayScreenUrl, {fromDay:modifyButtons.selectedDay});
			}
		}

		StandardButton {
			id: btnModifyProgram
			width: 95
			height: 36
			bottomClickMargin: 4
			anchors {
				bottom: btnCopyProgram.top
				bottomMargin: 8
				left: btnCopyProgram.left
			}

			text: p.modifyProgramBtnText
			onClicked: {
				modifyProgramDay(modifyButtons.selectedDay);
				app.startEditProgram(modifyButtons.selectedDay);
			}
		}
	}

	OnOffToggle {
		id: programToggle
		height: 36
		anchors.left: programSchedule.right
		anchors.leftMargin: 22
		anchors.verticalCenter: programSchedule.top
		anchors.verticalCenterOffset: 27

		fontColor: colors.spText
		rightTextOn: qsTr('On')
		rightTextOff:  qsTr('Off')
		isSwitchedOn: app.programEnabled
		leftIsSwitchedOn: false

		onSelectedChangedByUser: app.sendProgramState(isSwitchedOn);
	}

	/// displaying 4 temperature preset values, colors and names (away, sleep, home, confort) using a Repeater. Particular item's index has to be mapped to
	/// indexes used in dataset defined in ThermostatApp
	Item {
		id: programLegend

		width: programScreen.width - x
		height: 220
		anchors.left: programToggle.left
		anchors.bottom: programSchedule.bottom;

		Column {
			anchors.bottom: parent.bottom
			spacing: 16
			Repeater {
				id: legendRepeater
				model: app.thermStateName.length
				delegate: Item {
					width: programLegend.width
					height:  32

					Rectangle {
						id: modeColorRectangle
						width: 15; height: parent.height
						anchors.left: parent.left
						color: app.thermStateColor[legendRepeater.count - index - 1]
					}

					Text {
						id: modeNameText
						anchors.left: modeColorRectangle.right
						anchors.leftMargin: 8
						anchors.baseline: parent.top
						anchors.baselineOffset: 11
						font.family: qfont.semiBold.name
						font.pixelSize: 15
						color: colors.tpModeLabel
						text: app.thermStateName[legendRepeater.count - index - 1]
					}

					Text {
						id: modeTempText
						anchors.left: modeNameText.left
						anchors.baseline: parent.bottom
						font.family: qfont.regular.name
						font.pixelSize: 15
						color: colors.tpModeLabel
						text: p.formatTemp(p.getStateTemp(index));
					}
				}
			}
		}
	}

	StandardButton {
		id: btnSetTemp

		width: 95
		height: 36
		anchors.top: programLegend.bottom
		anchors.topMargin: 8
		anchors.left: programLegend.left
		text: qsTr("Set")

		onClicked: {
			stage.openFullscreen(app.temperaturePresetScreenUrl);
		}
	}
}
